/**
 * @copyright (c) Solution 360 GmbH
 * Handles sticky headers with removal/overscrolling of certain elements.
 * Also handles affix scrolling.
 * Also handles highlighting in vertical navbars because that, too, depends on sticky elements
 */
(function ($) {

    function s360_sticky_init() {
        $('.s360-sticky-placeholder').remove();
        $('.s360-sticky').each(function () {
            // add placeholder div beneath each potentially sticky element
            var $this = $(this);

            // check if the element is sticky on this viewport
            var breakpoints = $this.data('stickyBreakpoints');
            if(breakpoints) {
                var viewport = $('body').data('viewport');
                if(viewport && !breakpoints[viewport]) {
                        return;
                }
            }

            $this.removeClass('s360-sticky-active');
            $this.removeClass('s360-sticky-active-last');
            var id = _.uniqueId();
            $this.data('sticky-height', $this.outerHeight());
            $this.data('sticky-initial-y', $this.offset().top);
            $this.data('sticky-initial-css-top', $this.css('top'));

            // affixed elements now get their width set to a fixed value and limits for the distance they can scroll downwards
            if ($this.hasClass('s360-sticky-affix') && $this.is(':visible')) {
                $this.data('sticky-initial-css-width', $this.css('width'));
                $this.css('width', $this.outerWidth(false));
            } else {
                // normal sticky elements get a placeholder element below them
                $this.before('<div class="s360-sticky-placeholder" id="' + id + '"></div>');
                $this.data('sticky-placeholder-id', id);
            }
        });
    }

    function s360_sticky_update() {
        var offsetTop = 0;
        var stickyElementsCount = 0;
        var baseZIndex = 0;
        var scrollTop = $(window).scrollTop();
        // update sticky navigation
        $('.s360-sticky:visible').each(function () {
            var $this = $(this);

            // check if the element is sticky on this viewport
            var breakpoints = $this.data('stickyBreakpoints');
            if(breakpoints) {
                var viewport = $('body').data('viewport');
                if(viewport && !breakpoints[viewport]) {
                    return;
                }
            }

            $this.removeClass('s360-sticky-active-last');
            $this.data('sticky-height', $this.outerHeight()); // recalculate height (i.e. header might grow because of menu!)

            if ($this.hasClass('s360-sticky-affix')) {
                if ((scrollTop + offsetTop) >= $this.data('sticky-initial-y')) {
                    $this.addClass('s360-sticky-active');
                    var verticalLimit = 0;
                    var containerSelector = $this.data('affix-to');
                    if (containerSelector.length) {
                        var $container = $(containerSelector);
                        if ($container.length) {
                           verticalLimit =  $container.outerHeight(false) - $this.data('sticky-height');
                        }
                    }
                    if(verticalLimit <= 0) {
                        $this.removeClass('s360-sticky-active');
                    } else {
                        var verticalOffset = (scrollTop + offsetTop) - $this.data('sticky-initial-y');
                        if (verticalOffset >= verticalLimit) {
                            // absolute style display within parent
                            $this.css('top', verticalLimit + "px");
                            $this.addClass('s360-sticky-affix-at-limit')
                        } else {
                            // fixed style display - is smoother
                            $this.css('top', offsetTop + 'px');
                            $this.removeClass('s360-sticky-affix-at-limit');
                        }
                    }
                } else {
                    $this.removeClass('s360-sticky-active');
                    $this.css('top', $this.data('sticky-initial-css-top'));
                }
            } else {
                var id = $this.data('sticky-placeholder-id');
                var $placeholder = $('.s360-sticky-placeholder#' + id);

                if ((scrollTop + offsetTop) > $this.data('sticky-initial-y')) {
                    // make sticky!
                    $placeholder.css('padding-top', $this.data('sticky-height') + 'px');
                    $this.addClass('s360-sticky-active');
                    $this.css('top', offsetTop + 'px');
                    // auto adjust z-index to allow dropdowns to overlay "later" sticky elements
                    if (stickyElementsCount === 0) {
                        baseZIndex = $this.css('z-index');
                    } else {
                        $this.css('z-index', baseZIndex - stickyElementsCount);
                    }
                    offsetTop = offsetTop + $this.data('sticky-height');
                    stickyElementsCount++;
                } else {
                    // make unsticky
                    $this.removeClass('s360-sticky-active');
                    $placeholder.css('padding-top', '0');
                    $this.css('top', $this.data('sticky-initial-css-top'));
                    offsetTop = offsetTop - $this.data('sticky-height');
                }
            }
        });
        $('.s360-sticky-active:not(.s360-sticky-affix)').last().addClass('s360-sticky-active-last');

        /*
         Also: Update the position of the JTL Search Result box if it exists. (It does not work with
         sticky search inputs on its own).
         */
        var $jtlsearchresults = $('.jtl_search_results');
        if($jtlsearchresults.length) {
            var $input = $('.ac_input'); // this is the selector JTL search looks for in its inject.php
            if($input.length) {
                $input = $input.first();
                var newTop = $input.offset().top + $input.outerHeight();
                $jtlsearchresults.css('top', newTop + 'px');
            }
        }

        /*
            for that matter, we use the "s360-current-link" class and activate the first link that matches the top most visible anchor
         */
        var found = false;
        $('.s360-vertical-nav-anchor').each(function() {
            if(found) {
                return;
            }
            // notice we walk through the anchors from top to bottom. so the first one that is beyond our sticky threshold, is the one we want
            var $this = $(this);
            if($this.offset().top > (scrollTop + offsetTop) - 10) { // 10 px for imprecise heights
                found = true;
                $('.s360-vertical-nav a').removeClass('s360-current-link');
                $('.s360-vertical-nav a[href="#'+$this.attr('id')+'"]').addClass('s360-current-link');
            }
        });
    }


    // CALLS
    // ===============
    $(window).load(function () {
        s360_sticky_init();
        $(window).resize(_.debounce(function () {
            s360_sticky_init();
            s360_sticky_update();
        }, 150));
        // handle sticky elements within variation loaded content
        $('body').on('article-variation-loaded.s360', function() {
            s360_sticky_init();
            s360_sticky_update();
        });
        // handle sticky elements within checkout
        $(document).on('evo:loaded.evo.content', function() {
            s360_sticky_init();
            s360_sticky_update();
        });
        $(window).scroll(_.throttle(s360_sticky_update, 16));
    });
})(jQuery);
